@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2002 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,13 +15,22 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         misc.cpp  -  description
  *                         ------------------------
  *   begin                : Thu Nov 22 2001
- *   copyright            : (C) 2000-2002 by Damien Sandras
+ *   copyright            : (C) 2000-2003 by Damien Sandras
  *   description          : This file contains miscellaneous functions.
  *   Additional Code      : De Michele Cristiano, Miguel Rodrguez 
  *
@@ -30,21 +39,10 @@
 
 #include "../config.h"
 
-#include <ptlib.h>
-#include <esd.h>
-
-#ifndef DISABLE_GNOME
-#include <gnome.h>
-#else
-#include <gtk/gtk.h>
-#endif
-
-#include "config.h"
-#include "main_window.h"
-#include "common.h"
-#include "pref_window.h"
-#include "callbacks.h"
 #include "misc.h"
+#include "gnomemeeting.h"
+#include "callbacks.h"
+#include "config.h"
 #include "dialog.h"
 
 #include "../pixmaps/text_logo.xpm"
@@ -52,6 +50,8 @@
 
 /* Declarations */
 extern GtkWidget *gm;
+extern GnomeMeeting *MyApp;
+
 
 /* The functions */
 
@@ -59,65 +59,43 @@ extern GtkWidget *gm;
 void 
 gnomemeeting_threads_enter () 
 {
-  if (PThread::Current ()->GetThreadName () != "gnomemeeting") 
-  {    
-    //    cout << "Will take GDK Lock" << endl << flush;
-    PTRACE(1, "Will Take GDK Lock " << PThread::Current ()->GetThreadName ());
+  if ((PThread::Current () != NULL) && (PThread::Current ()->GetThreadName () != "gnomemeeting")) {    
     gdk_threads_enter ();
-    PTRACE(1, "GDK Lock Taken " << PThread::Current ()->GetThreadName ());
-    //    cout << "GDK Lock Taken" << endl << flush;
   }
-  else {
-
-    PTRACE(1, "Ignore GDK Lock Request : Main Thread");
-  }
-    
 }
 
 
 void 
 gnomemeeting_threads_leave () 
 {
-  if (PThread::Current ()->GetThreadName () != "gnomemeeting") {
+  if ((PThread::Current () != NULL) && (PThread::Current ()->GetThreadName () != "gnomemeeting")) {
 
-    //    cout << "Will Release GDK Lock" << endl << flush;
-    //    cout << PThread::Current ()->GetThreadName () << endl << flush;
-    PTRACE(1, "Will Release GDK Lock " << PThread::Current ()->GetThreadName ());
     gdk_threads_leave ();
-    PTRACE(1, "GDK Lock Released " << PThread::Current ()->GetThreadName ());
-    //    cout << "GDK Lock Released" << endl << flush;
   }
-  else {
-
-    PTRACE(1, "Ignore GDK UnLock request : Main Thread");
-  }
-    
 }
 
 
 GtkWidget *
-gnomemeeting_button (gchar *lbl, GtkWidget *pixmap)
+gnomemeeting_button_new (const char *lbl, 
+			 GtkWidget *pixmap)
 {
-  GtkWidget *button;
-  GtkWidget *hbox2;
-  GtkWidget *label;
+  GtkWidget *button = NULL;
+  GtkWidget *hbox2 = NULL;
+  GtkWidget *label = NULL;
   
   button = gtk_button_new ();
-  label = gtk_label_new (N_(lbl));
+  label = gtk_label_new_with_mnemonic (lbl);
   hbox2 = gtk_hbox_new (FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX (hbox2), pixmap, TRUE, TRUE, 0);  
   gtk_box_pack_start(GTK_BOX (hbox2), label, TRUE, TRUE, 0);
   
   gtk_container_add (GTK_CONTAINER (button), hbox2);
-    
-  gtk_widget_show (pixmap);
-  gtk_widget_show (label);
-  gtk_widget_show (hbox2);
-		
+
   return button;
 }
 
+
 void 
 gnomemeeting_log_insert (GtkWidget *text_view, gchar *text)
 {
@@ -161,7 +139,7 @@ void gnomemeeting_init_main_window_logo (GtkWidget *image)
   GdkPixbuf *text_logo_pix = NULL;
   GtkRequisition size_request;
 
-  GmWindow *gw = gnomemeeting_get_main_window (gm);
+  GmWindow *gw = MyApp->GetMainWindow ();
 
   gtk_widget_size_request (GTK_WIDGET (gw->video_frame), &size_request);
 
@@ -188,6 +166,19 @@ void gnomemeeting_init_main_window_logo (GtkWidget *image)
 
 /* This function overrides from a pwlib function */
 #ifndef STATIC_LIBS_USED
+static gboolean
+assert_error_msg (gpointer data)
+{
+  /* FIX ME: message */
+  gdk_threads_enter ();
+  gnomemeeting_error_dialog (GTK_WINDOW (gm), 
+			     _("Error: %s\n"), (gchar *) data);
+  gdk_threads_leave ();
+
+  return FALSE;
+}
+
+
 void 
 PAssertFunc (const char *file, int line, 
 	     const char *className, const char *msg)
@@ -201,10 +192,7 @@ PAssertFunc (const char *file, int line,
 
   cout << msg << endl << flush;
   
-  gnomemeeting_threads_enter ();
-  gnomemeeting_error_dialog (GTK_WINDOW (gm), 
-			     _("Error: %s\n"), msg);
-  gnomemeeting_threads_leave ();
+  g_idle_add (assert_error_msg, (gpointer) msg);
 
   inAssert = FALSE;
 
@@ -227,16 +215,21 @@ gnomemeeting_incoming_call_popup_new (gchar * utf8_name,
   GmWindow  *gw = NULL;
 
 
-  msg = g_strdup_printf (_("Call from %s\nusing %s"), (const char*) utf8_name, 
+
+  msg = g_strdup_printf (_("Call from %s\nusing %s"),
+			 (const char*) utf8_name,
 			 (const char *) utf8_app);
-  gw = gnomemeeting_get_main_window (gm);
+    
+  gw = MyApp->GetMainWindow ();
 
   widget = gtk_dialog_new ();
-  b1 = gtk_dialog_add_button (GTK_DIALOG (widget),
-			      _("Connect"), 0);
   b2 = gtk_dialog_add_button (GTK_DIALOG (widget),
-			      _("Disconnect"), 1);
-  
+			      _("Reject"), 0);
+  b1 = gtk_dialog_add_button (GTK_DIALOG (widget),
+			      _("Accept"), 1);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (widget), 1);
+
   label = gtk_label_new (msg);
   hbox = gtk_hbox_new (0, 0);
   
@@ -265,7 +258,6 @@ gnomemeeting_incoming_call_popup_new (gchar * utf8_name,
   
   gtk_window_set_transient_for (GTK_WINDOW (widget),
 				GTK_WINDOW (gm));
-  gtk_window_set_modal (GTK_WINDOW (widget), TRUE);
 
   gtk_widget_show_all (widget);
 
@@ -282,7 +274,7 @@ static int statusbar_clear_msg (gpointer data)
 
   gdk_threads_enter ();
 
-  gw = gnomemeeting_get_main_window (gm);
+  gw = MyApp->GetMainWindow ();
   id = gtk_statusbar_get_context_id (GTK_STATUSBAR (gw->statusbar),
 				     "statusbar");
 
@@ -422,189 +414,327 @@ gchar *gnomemeeting_get_utf8 (PString str)
 
 
 GtkWidget *
-gnomemeeting_table_add_entry (GtkWidget *table,        
-			      gchar *label_txt,        
-			      gchar *gconf_key,        
-			      gchar *tooltip,          
-			      int row)                 
-{                                                                              
-  GtkWidget *entry = NULL;                                                     
-  GtkWidget *label = NULL;                                                     
-  gchar *gconf_string = NULL;                                                  
-  GConfClient *client = NULL;                                                  
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);
-                                                                          
+gnomemeeting_table_add_entry (GtkWidget *table,
+			      gchar *label_txt,
+			      gchar *gconf_key,
+			      gchar *tooltip,
+			      int row,
+			      gboolean box)
+{
+  GValue value = { 0 };
+  int cols = 0;
+  GtkWidget *entry = NULL;
+  GtkWidget *label = NULL;
+  GtkWidget *hbox = NULL;
+  
+  gchar *gconf_string = NULL;
+
+  GConfClient *client = NULL;
+  GmPrefWindow *pw = NULL;
+
+  pw = MyApp->GetPrefWindow ();
   client = gconf_client_get_default ();
-                                                                               
 
-  label = gtk_label_new_with_mnemonic (label_txt);                                           
+  if (box) {
+    
+    hbox = gtk_hbox_new (FALSE, 0);
+    g_value_init (&value, G_TYPE_INT);
+    g_object_get_property (G_OBJECT (table), "n-columns", &value);
+    cols = g_value_get_int (&value);
+    g_value_unset (&value);
+  }
+  
+  label = gtk_label_new_with_mnemonic (label_txt);
 
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);                         
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);                
+  if (box)
+    gtk_box_pack_start (GTK_BOX (hbox), label,
+			FALSE, FALSE, GNOMEMEETING_PAD_SMALL * 2);
+  else
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,
+		      (GtkAttachOptions) (GTK_FILL),
+		      (GtkAttachOptions) (GTK_FILL),
+		      0, 0);
 
-                                                                               
-  entry = gtk_entry_new ();                                                    
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+
+  entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget ( GTK_LABEL(label), entry);
 
-  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gconf_string =  gconf_client_get_string (GCONF_CLIENT (client),              
-                                           gconf_key, NULL);                   
-                                                                               
-  if (gconf_string != NULL)                                                    
-    gtk_entry_set_text (GTK_ENTRY (entry), gconf_string);                      
-                                                                               
-  g_free (gconf_string);                                                       
-                                                                               
-                                                                               
+  if (box)
+    gtk_box_pack_start (GTK_BOX (hbox), entry,
+			FALSE, FALSE, GNOMEMEETING_PAD_SMALL * 2);
+  else
+    gtk_table_attach (GTK_TABLE (table), entry, 1, 2, row, row+1,
+		      (GtkAttachOptions) (NULL),
+		      (GtkAttachOptions) (NULL),
+		      0, 0);
+  
+  gconf_string =
+    gconf_client_get_string (GCONF_CLIENT (client), gconf_key, NULL);
+
+  if (gconf_string != NULL)
+    gtk_entry_set_text (GTK_ENTRY (entry), gconf_string);
+
+  g_free (gconf_string);
+
   /* We set the key as data to be able to get the data in order to block       
-     the signal in the gconf notifier */                             
+     the signal in the gconf notifier */
   g_object_set_data (G_OBJECT (entry), "gconf_key", (void *) gconf_key);
 
-  g_signal_connect (G_OBJECT (entry), "changed",                           
-		    G_CALLBACK (entry_changed),                         
+  g_signal_connect (G_OBJECT (entry), "changed", G_CALLBACK (entry_changed),
 		    (gpointer) g_object_get_data (G_OBJECT (entry),
 						  "gconf_key"));
 
+  if (box)
+    gtk_table_attach (GTK_TABLE (table), hbox, 0, cols, row, row+1,
+		      (GtkAttachOptions) (NULL),
+		      (GtkAttachOptions) (NULL),
+		      0, 0);
+        
   if (tooltip)
     gtk_tooltips_set_tip (pw->tips, entry, tooltip, NULL);
 
-  return entry;                                                                
+  return entry;
 }                                                                              
                                                                                
                                                                                
 GtkWidget *
-gnomemeeting_table_add_toggle (GtkWidget *table,       
-			       gchar *label_txt,       
-			       gchar *gconf_key,       
-			       gchar *tooltip,         
-			       int row, int col)       
+gnomemeeting_table_add_toggle (GtkWidget *table,
+			       gchar *label_txt,
+			       gchar *gconf_key,
+			       gchar *tooltip,
+			       int row)
 {
-  GtkWidget *toggle = NULL;  
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);
-  GConfClient *client = NULL;                                                  
-                                                                               
-  client = gconf_client_get_default ();                                        
+  GValue value = { 0 };
+  GtkWidget *toggle = NULL;
+  int cols = 0;
+  
+  GmPrefWindow *pw = NULL;
+  GConfClient *client = NULL;
+  
+  client = gconf_client_get_default ();
+  pw = MyApp->GetPrefWindow ();
 
+  g_value_init (&value, G_TYPE_INT);
+  g_object_get_property (G_OBJECT (table), "n-columns", &value);
+  cols = g_value_get_int (&value);
+  g_value_unset (&value);
+  
+  toggle = gtk_check_button_new_with_mnemonic (label_txt);
+  gtk_table_attach (GTK_TABLE (table), toggle, 0, cols, row, row+1,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (GTK_FILL),
+                    0, 0);
                                                                                
-  toggle = gtk_check_button_new_with_label (label_txt);                        
-  gtk_table_attach (GTK_TABLE (table), toggle, col, col+2, row, row+1,         
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), 
-				gconf_client_get_bool (client, 
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle),
+				gconf_client_get_bool (client,
 						       gconf_key, NULL));
-  gtk_tooltips_set_tip (pw->tips, toggle, tooltip, NULL);                     
+  gtk_tooltips_set_tip (pw->tips, toggle, tooltip, NULL);
 
   /* We set the key as data to be able to get the data in order to block       
-     the signal in the gconf notifier */                             
+     the signal in the gconf notifier */
   g_object_set_data (G_OBJECT (toggle), "gconf_key", (void *) gconf_key);
-                                                                               
+
   g_signal_connect (G_OBJECT (toggle), "toggled", G_CALLBACK (toggle_changed),
-		    (gpointer) gconf_key);                                   
-                                                                               
+		    (gpointer) gconf_key);
+
   return toggle;
 }                                                                              
 
 
 GtkWidget *
 gnomemeeting_table_add_spin (GtkWidget *table,       
-			     gchar *label_txt,       
+			     gchar *label_txt,
 			     gchar *gconf_key,       
-			     gchar *tooltip,         
-			     double min, double max, double step, int row)
+			     gchar *tooltip,
+			     double min,
+			     double max,
+			     double step,
+			     int row,
+			     gchar *label_txt2,
+			     gboolean box)
 {
+  GtkWidget *hbox = NULL;
   GtkAdjustment *adj = NULL;
   GtkWidget *label = NULL;
   GtkWidget *spin_button = NULL;
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);
+  GmPrefWindow *pw = NULL;
+  
+  GConfClient *client = NULL;
 
-  GConfClient *client = NULL;                                                  
-                                                                               
-  client = gconf_client_get_default ();                                        
+  client = gconf_client_get_default ();
+  pw = MyApp->GetPrefWindow ();
 
+  if (box)
+    hbox = gtk_hbox_new (FALSE, 0);
+  
+  label = gtk_label_new_with_mnemonic (label_txt);
 
-  label = gtk_label_new (label_txt);                                           
-
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);                         
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);                
-
+  if (box)
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE,
+			GNOMEMEETING_PAD_SMALL * 2);
+  else
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,
+		      (GtkAttachOptions) (GTK_FILL),
+		      (GtkAttachOptions) (GTK_FILL),
+		      0, 0);
 
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+  
   adj = (GtkAdjustment *) 
-    gtk_adjustment_new (gconf_client_get_int (client, gconf_key, 0), min, max, step, 
+    gtk_adjustment_new (gconf_client_get_int (client, gconf_key, 0),
+			min, max, step,
 			2.0, 1.0);
+  spin_button = gtk_spin_button_new (adj, 1.0, 0);
 
-  spin_button =
-    gtk_spin_button_new (adj, 1.0, 0);
-                                                                               
-  gtk_table_attach (GTK_TABLE (table), spin_button, 1, 2, row, row+1,         
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gtk_adjustment_set_value (GTK_ADJUSTMENT (adj), 
-			    gconf_client_get_int (client, gconf_key, NULL));
+  if (box)
+    gtk_box_pack_start (GTK_BOX (hbox), spin_button, FALSE, FALSE,
+			GNOMEMEETING_PAD_SMALL * 2);
+  else
+    gtk_table_attach (GTK_TABLE (table), spin_button, 1, 2, row, row+1,
+		      (GtkAttachOptions) (GTK_FILL),
+		      (GtkAttachOptions) (GTK_FILL),
+		      0, 0);
+
+  if (box && label_txt2) {
+    
+    label = gtk_label_new_with_mnemonic (label_txt2);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE,
+			GNOMEMEETING_PAD_SMALL * 2);
+  }
+
+  if (box)
+    gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, row, row+1,
+		      (GtkAttachOptions) (GTK_FILL),
+		      (GtkAttachOptions) (GTK_FILL),
+		      0, 0);
 
-                                                                               
   gtk_tooltips_set_tip (pw->tips, spin_button, tooltip, NULL);     
 
   /* We set the key as data to be able to get the data in order to block       
-     the signal in the gconf notifier */                             
+     the signal in the gconf notifier */
   g_object_set_data (G_OBJECT (adj), "gconf_key", (void *) gconf_key);
                                                                                
-  g_signal_connect (G_OBJECT (adj), "value-changed", G_CALLBACK (adjustment_changed),
-		    (gpointer) gconf_key);                                   
-                                                                               
+  g_signal_connect (G_OBJECT (adj), "value-changed",
+		    G_CALLBACK (adjustment_changed),
+		    (gpointer) gconf_key);
+
   return spin_button;
+}
+
+
+void
+gnomemeeting_table_add_spin_range (GtkWidget *table,
+				   gchar *label1_txt,
+				   GtkWidget **spin1,
+				   gchar *label2_txt,
+				   GtkWidget **spin2,
+				   gchar *label3_txt,
+				   gchar *spin1_gconf_key,
+				   gchar *spin2_gconf_key,
+				   gchar *spin1_tooltip,
+				   gchar *spin2_tooltip,
+				   double spin1_min,
+				   double spin2_min,
+				   double spin1_max,
+				   double spin2_max,
+				   double spins_step,
+				   int row)
+{
+  int val1 = 0, val2 = 0;
+  GtkWidget *hbox = NULL;
+  GtkAdjustment *adj1 = NULL;
+  GtkAdjustment *adj2 = NULL;
+  GtkWidget *label = NULL;
+
+  GmPrefWindow *pw = NULL;
+  GConfClient *client = NULL;
+
+  pw = MyApp->GetPrefWindow ();
+  client = gconf_client_get_default ();
+
+  hbox = gtk_hbox_new (FALSE, 0);
+  label = gtk_label_new_with_mnemonic (label1_txt);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE,
+		      GNOMEMEETING_PAD_SMALL * 2);
+  
+  val1 = gconf_client_get_int (client, spin1_gconf_key, 0);
+  adj1 = (GtkAdjustment *) 
+    gtk_adjustment_new (val1, spin1_min, spin1_max, spins_step, 2.0, 1.0);
+  *spin1 = gtk_spin_button_new (adj1, 1.0, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), *spin1, FALSE, FALSE,
+		      GNOMEMEETING_PAD_SMALL * 2);
+
+  label = gtk_label_new_with_mnemonic (label2_txt);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE,
+		      GNOMEMEETING_PAD_SMALL * 2);
+
+  val2 = gconf_client_get_int (client, spin2_gconf_key, 0);
+  adj2 = (GtkAdjustment *) 
+    gtk_adjustment_new (val2, spin2_min, spin2_max, spins_step, 2.0, 1.0);
+  *spin2 = gtk_spin_button_new (adj2, 1.0, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), *spin2, FALSE, FALSE,
+		      GNOMEMEETING_PAD_SMALL * 2);
+  
+  label = gtk_label_new_with_mnemonic (label3_txt);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE,
+		      GNOMEMEETING_PAD_SMALL * 2);
+
+  gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, row, row+1,
+                    (GtkAttachOptions) (NULL),
+		    (GtkAttachOptions) (NULL),
+		    0, 0);
+  
+  gtk_tooltips_set_tip (pw->tips, *spin1, spin2_tooltip, NULL);
+  gtk_tooltips_set_tip (pw->tips, *spin2, spin2_tooltip, NULL);
+  
+  
+  /* We set the key as data to be able to get the data in order to block       
+     the signal in the gconf notifier */
+  g_object_set_data (G_OBJECT (adj1), "gconf_key", (void *) spin1_gconf_key);
+  g_object_set_data (G_OBJECT (adj2), "gconf_key", (void *) spin2_gconf_key);
+
+  g_signal_connect (G_OBJECT (adj1), "value-changed",
+		    G_CALLBACK (adjustment_changed),
+		    (gpointer) spin1_gconf_key);
+  g_signal_connect (G_OBJECT (adj2), "value-changed",
+		    G_CALLBACK (adjustment_changed),
+		    (gpointer) spin2_gconf_key);
 }                                                                              
 
 
 GtkWidget *
-gnomemeeting_table_add_int_option_menu (GtkWidget *table,       
-					gchar *label_txt, 
+gnomemeeting_table_add_int_option_menu (GtkWidget *table,
+					gchar *label_txt,
 					gchar **options,
-					gchar *gconf_key,       
-					gchar *tooltip,         
-					int row)       
+					gchar *gconf_key,
+					gchar *tooltip,
+					int row)
 {
   GtkWidget *item = NULL;
-  GtkWidget *label = NULL;                                                     
+  GtkWidget *label = NULL;
   GtkWidget *option_menu = NULL;
   GtkWidget *menu = NULL;
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);
+  GmPrefWindow *pw = NULL;
 
-  int cpt = 0;                                                   
+  int cpt = 0;
 
   GConfClient *client = NULL;                                                  
-                                                                               
+  
   client = gconf_client_get_default ();                                        
+  pw = MyApp->GetPrefWindow ();
 
+  label = gtk_label_new_with_mnemonic (label_txt);
 
-  label = gtk_label_new (label_txt);                                           
-
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);                         
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);                
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (GTK_FILL),
+                    0, 0);
 
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
   menu = gtk_menu_new ();
   option_menu = gtk_option_menu_new ();
@@ -618,25 +748,24 @@ gnomemeeting_table_add_int_option_menu (GtkWidget *table,
   }
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 
+  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu),
  			       gconf_client_get_int (client, gconf_key, NULL));
 
-  gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,         
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
-                                                                               
-                                                                               
-  gtk_tooltips_set_tip (pw->tips, option_menu, tooltip, NULL);     
+  gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (GTK_FILL),
+                    0, 0);
 
-  /* We set the key as data to be able to get the data in order to block       
-     the signal in the gconf notifier */                             
+  gtk_tooltips_set_tip (pw->tips, option_menu, tooltip, NULL);
+
+  /* We set the key as data to be able to get the data in order to block
+     the signal in the gconf notifier */
   g_object_set_data (G_OBJECT (option_menu), "gconf_key", (void *) gconf_key);
-                                                                               
-  g_signal_connect (G_OBJECT (GTK_OPTION_MENU (option_menu)->menu), 
+
+  g_signal_connect (G_OBJECT (GTK_OPTION_MENU (option_menu)->menu),
 		    "deactivate", G_CALLBACK (int_option_menu_changed),
-  		    (gpointer) gconf_key);                                   
-                                                                               
+  		    (gpointer) gconf_key);
+
   return option_menu;
 }                                                                              
 
@@ -653,7 +782,7 @@ gnomemeeting_table_add_string_option_menu (GtkWidget *table,
   GtkWidget *label = NULL;                                                     
   GtkWidget *option_menu = NULL;
   GtkWidget *menu = NULL;
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);  
+  GmPrefWindow *pw = MyApp->GetPrefWindow ();  
   gchar *gconf_string = NULL;
   int history = -1;
 
@@ -667,12 +796,12 @@ gnomemeeting_table_add_string_option_menu (GtkWidget *table,
   label = gtk_label_new (label_txt);                                           
 
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (GTK_FILL),
+                    0, 0);
                                                                                
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);                         
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);                
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);                         
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
 
   menu = gtk_menu_new ();
@@ -704,10 +833,10 @@ gnomemeeting_table_add_string_option_menu (GtkWidget *table,
  			       history);
 
 
-  gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,         
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
+  gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,
+                    (GtkAttachOptions) (GTK_FILL),                
+                    (GtkAttachOptions) (GTK_FILL),                
+                    0, 0);           
                                                                                
                            
   gtk_tooltips_set_tip (pw->tips, option_menu, tooltip, NULL);
@@ -739,7 +868,7 @@ gnomemeeting_table_add_pstring_option_menu (GtkWidget *table,
   GtkWidget *label = NULL;                                                     
   GtkWidget *option_menu = NULL;
   GtkWidget *menu = NULL;
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);  
+  GmPrefWindow *pw = MyApp->GetPrefWindow ();  
   gchar *gconf_string = NULL;
   int history = -1;
 
@@ -750,15 +879,15 @@ gnomemeeting_table_add_pstring_option_menu (GtkWidget *table,
   client = gconf_client_get_default ();                                        
 
 
-  label = gtk_label_new (label_txt);                                           
+  label = gtk_label_new_with_mnemonic (label_txt);
 
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row+1,                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
+                    (GtkAttachOptions) (GTK_FILL),                
+                    (GtkAttachOptions) (GTK_FILL),                
+                    0, 0);           
                                                                                
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);                         
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);                
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);                         
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);                
 
 
   menu = gtk_menu_new ();
@@ -791,9 +920,9 @@ gnomemeeting_table_add_pstring_option_menu (GtkWidget *table,
 
 
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, row, row+1,         
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    (GtkAttachOptions) (GTK_FILL | GTK_SHRINK),                
-                    GNOMEMEETING_PAD_SMALL, GNOMEMEETING_PAD_SMALL);           
+                    (GtkAttachOptions) (GTK_FILL),                
+                    (GtkAttachOptions) (GTK_FILL),                
+                    0, 0);           
                                                                                
                            
   gtk_tooltips_set_tip (pw->tips, option_menu, tooltip, NULL);
@@ -875,7 +1004,8 @@ GtkWidget *
 gnomemeeting_vbox_add_table (GtkWidget *vbox,         
 			     gchar *frame_name,       
 			     int rows, int cols)      
-{                                                                              
+{                                                         
+  GtkWidget *hbox = NULL;
   GtkWidget *frame = NULL;
   GtkWidget *table = NULL;
   GtkWidget *label = NULL;
@@ -883,7 +1013,8 @@ gnomemeeting_vbox_add_table (GtkWidget *vbox,
   PangoAttrList *attrs = NULL;
   PangoAttribute *attr = NULL;
    
-  
+  hbox = gtk_hbox_new (FALSE, 6);
+
   frame = gtk_frame_new (frame_name);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
   
@@ -902,12 +1033,17 @@ gnomemeeting_vbox_add_table (GtkWidget *vbox,
                                                                               
   table = gtk_table_new (rows, cols, FALSE);                                   
                                                                               
-  gtk_container_add (GTK_CONTAINER (frame), table); 
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 4);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 4); 
+  gtk_container_add (GTK_CONTAINER (frame), hbox); 
+
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
+
+  label = gtk_label_new ("    ");
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), table, FALSE, FALSE, 0);
                                                                                
-  gtk_table_set_row_spacings (GTK_TABLE (table), GNOMEMEETING_PAD_SMALL);     
-  gtk_table_set_col_spacings (GTK_TABLE (table), GNOMEMEETING_PAD_SMALL);     
+  gtk_table_set_row_spacings (GTK_TABLE (table), 2);     
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);     
   
   return table;
 }                                                                              
